package org.sonatype.tycho.p2.publisher;

import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Properties;

import org.codehaus.plexus.util.IOUtil;
import org.codehaus.plexus.util.StringUtils;
import org.sonatype.tycho.p2.publisher.model.InstallableUnit;

/**
 * JAVADOC
 * 
 * @author Alin Dreghiciu
 */
public class P2Tasks
{

    public static final String P2_PROFILE = "tycho-p2";

    public static void writeInstallerProperties( final File file, final List<InstallableUnit> rootUnits )
        throws Exception
    {
        OutputStream outputStream = null;
        try
        {
            if ( file.getParentFile() != null )
            {
                file.getParentFile().mkdirs();
            }
            outputStream = new FileOutputStream( file );

            final List<String> roots = new ArrayList<String>();
            for ( final InstallableUnit unit : rootUnits )
            {
                roots.add( unit.getId() );
            }

            final Properties properties = new Properties();
            properties.setProperty( "eclipse.p2.roots", StringUtils.join( roots.iterator(), "," ) );
            properties.setProperty( "eclipse.p2.metadata", "file:" + file.getParent() );
            properties.setProperty( "eclipse.p2.artifacts", "file:" + file.getParent() );
            properties.setProperty( "eclipse.p2.flavor", "tooling" );
            properties.setProperty( "eclipse.p2.profileName", P2_PROFILE );

            properties.store( outputStream, "P2 properties" );
        }
        finally
        {
            IOUtil.close( outputStream );
        }
    }

    public static void writeAgentScript( final File file, final List<InstallableUnit> rootUnits,
                                         final Collection<String> repositories )
        throws Exception
    {
        PrintWriter writer = null;
        try
        {
            if ( file.getParentFile() != null )
            {
                file.getParentFile().mkdirs();
            }
            writer = new PrintWriter( new FileWriter( file ) );

            final List<String> roots = new ArrayList<String>();
            for ( final InstallableUnit unit : rootUnits )
            {
                roots.add( unit.getId() );
            }

            final StringBuilder repos = new StringBuilder();
            repos.append( "file:" ).append( file.getParent() );
            for ( final String repo : repositories )
            {
                repos.append( "," ).append( repo );
            }

            writer.println( "./eclipse \\" );
            writer.println( "-application org.eclipse.equinox.p2.director \\" );
            writer.println( "-installIU " + StringUtils.join( roots.iterator(), "," ) + " \\" );
            writer.println( "-repository " + repos.toString() + " \\" );
            writer.println( "-profile " + P2_PROFILE + " \\" );
            writer.println( "-profileproperties org.eclipse.equinox.p2.planner.resolveMetaRequirements=false \\" );
            writer.println( "-destination $1 \\" );
            writer.println( "-consolelog" );

            writer.flush();
        }
        finally
        {
            IOUtil.close( writer );
        }
    }

    public static String[] getDirectorParameters( final File destination, final File updateSite,
                                                  final List<InstallableUnit> rootUnits,
                                                  final Collection<String> repositories, final String tag )
    {
        final List<String> roots = new ArrayList<String>();
        for ( final InstallableUnit unit : rootUnits )
        {
            roots.add( unit.getId() );
        }

        final StringBuilder repos = new StringBuilder();
        repos.append( "file:" ).append( updateSite.getAbsolutePath() );
        for ( final String repo : repositories )
        {
            repos.append( "," ).append( repo );
        }

        final StringBuilder profileProperties = new StringBuilder();
        profileProperties.append( "org.eclipse.equinox.p2.planner.resolveMetaRequirements=false" );

        return new String[] { "-installIU", StringUtils.join( roots.iterator(), "," ), "-repository", repos.toString(),
            "-profile", P2_PROFILE, "-profileproperties", profileProperties.toString(), "-destination",
            destination.getAbsolutePath(), "-tag", tag };
    }
}
