package org.sonatype.tycho.p2.publisher.impl;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;

import org.codehaus.plexus.component.annotations.Component;
import org.codehaus.plexus.util.IOUtil;
import org.sonatype.aether.artifact.Artifact;
import org.sonatype.aether.graph.DependencyNode;
import org.sonatype.aether.util.filter.PatternExclusionsDependencyFilter;
import org.sonatype.aether.util.graph.CloningDependencyVisitor;
import org.sonatype.aether.util.graph.FilteringDependencyVisitor;
import org.sonatype.aether.util.graph.PreorderNodeListGenerator;
import org.sonatype.aether.util.version.GenericVersionScheme;
import org.sonatype.tycho.p2.publisher.NoOpPublisherLifecycleParticipant;
import org.sonatype.tycho.p2.publisher.PublisherLifecycleParticipant;
import org.sonatype.tycho.p2.publisher.model.Advice;
import org.sonatype.tycho.p2.publisher.model.AppliesTo;
import org.sonatype.tycho.p2.publisher.model.Assembly;
import org.sonatype.tycho.p2.publisher.model.io.xpp3.AssemblyXpp3Reader;

@Component( role = PublisherLifecycleParticipant.class, hint = "artifactAdviceExtractor" )
public class ArtifactAdviceExtractor
    extends NoOpPublisherLifecycleParticipant
{

    @Override
    public Collection<Advice> getAdvices( final DependencyNode root )
    {
        final PreorderNodeListGenerator collector = new PreorderNodeListGenerator();
        root.accept( collector );
        final Collection<Advice> advices = new ArrayList<Advice>();
        for ( final Artifact artifact : collector.getArtifacts( false ) )
        {
            if ( "advice".equals( artifact.getExtension() ) )
            {
                advices.addAll( extractAdvices( artifact.getFile(), Utils.fullArtifactId( root.getDependency().getArtifact() ) ) );
            }
        }
        return advices;
    }

    private Collection<Advice> extractAdvices( final File artifactFile, final String id )
    {
        final Collection<Advice> advices = new ArrayList<Advice>();
        InputStream is = null;
        try
        {
            is = new BufferedInputStream( new FileInputStream( artifactFile ) );
            final Assembly assembly = new AssemblyXpp3Reader().read( is );
            if ( assembly != null )
            {
                final List<Advice> assemblyAdvices = assembly.getAdvices();
                addDefaults( assemblyAdvices, id );
                advices.addAll( assemblyAdvices );
            }
        }
        catch ( final Exception e )
        {
            throw new RuntimeException( e );
        }
        finally
        {
            IOUtil.close( is );
        }
        return advices;
    }

    private void addDefaults( final Collection<Advice> advices, final String id )
    {
        for ( final Advice advice : advices )
        {
            AppliesTo appliesTo = advice.getAppliesTo();
            if ( appliesTo == null || ( appliesTo.getIncludes().isEmpty() && appliesTo.getExcludes().isEmpty() ) )
            {
                appliesTo = new AppliesTo();
                appliesTo.setIncludes( Arrays.asList( id ) );
                advice.setAppliesTo( appliesTo );
            }
        }
    }

    @Override
    public DependencyNode cleanupDependencyTree( final DependencyNode root )
    {
        final CloningDependencyVisitor clonningVisitor = new CloningDependencyVisitor();

        final PatternExclusionsDependencyFilter nodeFilter =
            new PatternExclusionsDependencyFilter( new GenericVersionScheme(), "*:*:advice" );

        final FilteringDependencyVisitor filter = new FilteringDependencyVisitor( clonningVisitor, nodeFilter );
        root.accept( filter );

        return clonningVisitor.getRootNode();
    }
}
